unit uMHA_Instrument_OPS;

interface

uses
  Contnrs,
  Classes,
  SysUtils,
  fIA_dialogs, Dialogs,
  uIA_Instrument;

type
  TInstrument_OPS = class
  private
    FListOfInstruments: TObjectList;
  protected
  public
    constructor Create;
    destructor Destroy; override;

    procedure InitializeListOfAnswers(var aInstrument: TInstrument);
    function Load(var aInstrument: TInstrument): Boolean;
    function LoadDescriptiveProperties(var aInstrument: TInstrument): Boolean;
    function LoadQuestionAndIntroLists(var aInstrument: TInstrument): Boolean;
    function LoadQuestionChoices(var aInstrument: TInstrument): Boolean;
    procedure LoadSectionList(var aInstrument: TInstrument);
    procedure LoadScales(var aInstrument: TInstrument);
    procedure LoadRuleList(var aInstrument: TInstrument);
    procedure LoadKeys(var aInstrument: TInstrument);

    function ReadInstrumentDetails(aInstrumentName: string): string;
    function LoadListOfAvailableInstruments(aUserKeys: string): string;

    property ListOfInstruments: TObjectList read FListOfInstruments write FListOfInstruments;
  end; // TInstrument_OPS

var

  Instrument_OPS              : TInstrument_OPS;

implementation

uses
  uIA_Const,
  uMHA_Const,
  uIA_Utils,
  uIA_Instrument_IO,
  uMHA_Instrument_IO,
  uMHA_ResourceFile_IO,
  uMHA_LocalFiles,
  uIA_Question,
  uIA_Question_IO,
  uIA_MultiChoice_IO,
  uIA_Rule_IO,
  uIA_SectionLabel_IO,
  uIA_Scales_IO,
  uIA_Keys_IO,
  uIA_Answer,
  uMHA_DM;


constructor TInstrument_OPS.Create;
begin
  inherited Create;
  FListOfInstruments := TObjectList.Create;
end;

destructor TInstrument_OPS.Destroy;
begin
  ListOfInstruments.Free;
  inherited Destroy;
end;

procedure TInstrument_OPS.InitializeListOfAnswers(var aInstrument: TInstrument);
var
  i                           : Integer;
  AAnswer                     : TAnswer;
begin
  aInstrument.ListOfAnswers.Clear;

  for i := 0 to aInstrument.ListOfQuestions.Count - 1 do
    begin
      AAnswer := TAnswer.Create;
      AAnswer.QuestionID := TQuestion(aInstrument.ListOfQuestions[i]).QuestionID;
      AAnswer.AnswerValue := NOT_ASKED;
      AAnswer.AnswerText := NOT_ASKED;
      AAnswer.IsMultiChoice := False;
      AAnswer.Designator := TQuestion(aInstrument.ListOfQuestions[i]).Designator;
      aInstrument.ListOfAnswers.Add(AAnswer);
    end; // for i
end;

function TInstrument_OPS.LoadDescriptiveProperties(var aInstrument: TInstrument): Boolean;
begin
  Result := False;

  if InstrumentReadDescriptiveProperties(DataModule1.RPCBroker1, aInstrument) = BAD_DATA then
    begin
      ShowMessage('Unable to load descriptive properties of ' + aInstrument.InstrumentName + '.');
      Exit;
    end
  else
    Result := True;

end; //LoadDescriptiveProperties


function TInstrument_OPS.Load(var aInstrument: TInstrument): BOOLEAN;
begin
  Result := True; //the default

  LoadDescriptiveProperties(aInstrument);

  if not LoadQuestionAndIntroLists(aInstrument) or
    not LoadQuestionChoices(aInstrument) then
    begin
      Result := False;
      Exit;
    end;

  LoadRuleList(aInstrument);
  LoadSectionList(aInstrument);
  LoadScales(aInstrument);
  LoadKeys(aInstrument);
  InitializeListOfAnswers(aInstrument);

end;

function TInstrument_OPS.LoadQuestionAndIntroLists(var aInstrument: TInstrument): Boolean;
begin
  Result := False;

  aInstrument.ListOfQuestions.Clear;
  aInstrument.ListOfIntroductions.Clear;

  // See uIA_Question_IO
  if ReadQuestionAndIntroLists(DataModule1.RPCBroker1, aInstrument) = OK then
    Result := True
  else
    ShowMessage('Unable to load questions and introductions for ' + aInstrument.InstrumentName + '.');
end;


function TInstrument_OPS.LoadQuestionChoices(var aInstrument: TInstrument): Boolean;
// This adds the multiple choice items to each question in the ListOfQuestions,
// although some questions will not need this service.
var
  i, j                        : Integer;
  TempStr                     : string;
  TempList                    : TStringList;

begin
  Result := True;
  TempList := TStringList.Create;

  PiecesToList(ReadChoiceList(aInstrument.InstrumentName, DataModule1.RPCBroker1), RECORD_END, TempList);

// Raw data from Vista looks like:
// 469^1^100098^One^^1
// 469^2^100099^Two^^2
// 469^3^100100^Three^^3
// 469^4^100101^Four^^4
// 480^1^100165^Low^
// 480^2^100166^Medium^
// 480^3^100752^ high^
// 766^1^237^No^N
// 766^2^241^Yes^Y
// 766^3^1251^Declined to answer^
// 766^4^1252^Failed to ascertain^
// 777^1^1260^Very irregular answer^
// 777^2^1261^Second irregular answer^
// 777^3^1262^Unique response^
// 777^4^1263^Very unusual response^

  if (TempList[0] = BAD_DATA) then
    begin
      ShowMessage('Unable to load question choices for ' + aInstrument.InstrumentName + '.');
      Result := False;
      TempList.Free;
      Exit;
    end;

  for i := 0 to aInstrument.ListOfQuestions.Count - 1 do
    begin
      for j := 0 to TempList.count - 1 do
        begin
          if TQuestion(aInstrument.ListOfQuestions[i]).ChoiceTypeID =
            Piece(TempList[j], U, 1) then
            begin

              TempStr := Piece(TempList[j], U, 2) + U; // Designator/Sequence
              TempStr := TempStr + Piece(TempList[j], U, 3) + U; // ChoiceID
              TempStr := TempStr + Piece(TempList[j], U, 4) + U; // Text
              TempStr := TempStr + Piece(TempList[j], U, 5); // Legacy Value

              TQuestion(aInstrument.ListOfQuestions[i]).MultiChoice.Choices.Add(TempStr);
              TQuestion(aInstrument.ListOfQuestions[i]).MultiChoice.NumberOfPicks :=
                TQuestion(aInstrument.ListOfQuestions[i]).qMax;
              TQuestion(aInstrument.ListOfQuestions[i]).MultiChoice.NumberOfChoices :=
                TQuestion(aInstrument.ListOfQuestions[i]).MultiChoice.NumberOfChoices + 1;
            end;

        end; // for j

    end; // for i

  TempList.Free;

end;

procedure TInstrument_OPS.LoadRuleList(var aInstrument: TInstrument);
// This adds the business rule to the ListOfRules.
begin
  aInstrument.ListOfRules.Clear;

  if not ReadRules(DataModule1.RPCBroker1, aInstrument) then // uIA_Rule_IO
    ShowMessage('Unable to load rules for ' + aInstrument.InstrumentName + '.');
end;

procedure TInstrument_OPS.LoadSectionList(var aInstrument: TInstrument);
begin
  aInstrument.ListOfSections.Clear;

  // See uIA_SectionLabel_IO
  if not ReadSectionList(DataModule1.RPCBroker1, False {NeedIEN}, aInstrument) then
    ShowMessage('Unable to load section labels for ' + aInstrument.InstrumentName + '.');
end;

procedure TInstrument_OPS.LoadScales(var aInstrument: TInstrument);
begin
  aInstrument.ListOfScales.Clear;

  // See uIA_Scales_IO
  if not ReadScales(DataModule1.RPCBroker1, aInstrument) then
    ShowMessage('Unable to load scales for ' + aInstrument.InstrumentName + '.');
end;

procedure TInstrument_OPS.LoadKeys(var aInstrument: TInstrument);
begin
  aInstrument.ListOfKeys.Clear;

  if not ReadScoringKeys(DataModule1.RPCBroker1, aInstrument) then // uIA_Keys_IO
    ShowMessage('Unable to load the scoring keys for ' + aInstrument.InstrumentName);
end;

function TInstrument_OPS.ReadInstrumentDetails(aInstrumentName: string): string;
var
  tmpFileName                 : string;
  sr                          : TSearchRec;
// Yields string with DaysToRestart: integer (Range is -1 to 99
//  where -1 is no limit, 0 is not allowed) ^
// GenerateProgNote: Boolean ^
// ProgNoteTitle: string ^
// ConsultNoteTitle: string
begin
  Result := '';

  // Check the resource file first
  if (FindFirst(LocalFiles.InstrumentFilesPath + '\' +
    aInstrumentName + FILE_NAME_DELIMITER + '*.' + RESOURCE_FILE_EXT,
    faAnyFile, sr) = 0) then
    begin
      tmpFileName := LocalFiles.InstrumentFilesPath + '\' + sr.Name;
      FindClose(sr);

      try
        begin
          Result := ResourceFile_IO.ReadInstrumentDetailsFromFile(TmpFileName);
          Exit;
        end;
      except
        //If you like, you can tell the user "I cant read options"
      end
    end; //if (FindFirst...

  // File not there; try Vista
  if (result = BAD_DATA) and (DataBaseMode = dbVista) then
    begin
      result := ReadInstrumentDetailsVista(aInstrumentName, DataModule1.RPCBroker1);
      // Range is -1 to 99 where -1 is no limit, 0 is not allowed.
      Exit;
    end;

  // Vista not there; use default
  Result := '2^Yes^'+ PROGRESS_NOTE_TITLE + '^' + CONSULT_NOTE_TITLE;
end;


function TInstrument_OPS.LoadListOfAvailableInstruments(aUserKeys: string): string;
begin
  // see Instrument_IO
  Result := ReadListOfAvailableInstruments(aUserKeys, DataModule1.RPCBroker1);
end;


initialization
  Instrument_OPS := TInstrument_OPS.Create;

finalization
  Instrument_OPS.Free;

end.

