unit uMHA_ResourceFile_IO;

(*
Resource files are used to speed operation. When an instrument is called for
display for data-entry, the resource file is quicker to provide Questions,
Answers, Introductions, Sections and Rules than a broker call.

Scale Groups, Scales and Keys are not needed in the resource file: This are
for results only and broker are used for that information.

The format of these files is NOT the same as import/export files.

*)

interface


uses
  Windows,
  Messages,
  SysUtils,
  System.Contnrs,
  System.UITypes,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  uIA_Question,
  uIA_Instrument,
  uIA_Introduction,
  uIA_SectionLabel,
  StdCtrls,
  ComCtrls,
  Dialogs;

type
  TResourceFile_IO = class

  private
    { Private declarations }

    procedure WriteQuestions(aInstrument: TInstrument);
    procedure WriteIntroductions(aInstrument: TInstrument);
    procedure WriteSections(aInstrument: TInstrument);
    procedure WriteRules(aInstrument: TInstrument);

    procedure ReadDescriptiveProperties(aInstrument: TInstrument);
    procedure ReadQuestions(aInstrument: TInstrument);
    procedure ReadIntroductions(aInstrument: TInstrument);
    procedure ReadSections(aInstrument: TInstrument);
    procedure ReadRules(aInstrument: TInstrument);
    procedure UpdateLatestVersion(DLL_Ver: string);
  public
    { Public declarations }
    procedure BuildInstrumentResourceFile(FullFileName: string;
      aInstrument: TInstrument);
    procedure ReadInstrumentResourceFile(FullFileName: string; var aInstrument: TInstrument);
    function ReadInstrumentDetailsFromFile(FullFileName: string): string;
  end;

var
  ResourceFile_IO             : TResourceFile_IO;

implementation

uses
  uIA_Const,
  uMHA_Const,
  uMHA_Registry,
  uIA_Utils,
  uIA_Rule,
  uIA_Display,
  uIA_ScaleGroup,
  uIA_Scale,
  uIA_Key,
  uIA_Display_OPS,
  uIA_Question_OPS;


var
  lvStringList                : TStringList;
  lvFont                      : TFont;
  lvStartRule,
    lvStartIntro,
    lvStartSection            : Integer;

procedure TResourceFile_IO.UpdateLatestVersion(DLL_Ver: string);
var
  SavedInt,
    ThisInt,
    i                         : integer;
begin
  // Make sure DLL_Ver is in proper format
  if NumberOfPieces(DLL_Ver, '.') <> 3 then
    exit;

  for i := 1 to Length(DLL_Ver) do
    if CharInSet(DLL_Ver[i], ['0'..'9']) then
    // do nothing
    else
      exit;

  for i := 1 to 4 do
    begin
      SavedInt := StrToIntDef(Piece(gvDll_Version, '.', i), 0);
      ThisInt := StrToIntDef(Piece(DLL_Ver, '.', i), 0);

      if ThisInt > SavedInt then
        begin
          gvDLL_Version := DLL_Ver;
          MHA_Registry.SaveRegistry;
          break;
        end;
    end;
end;


procedure TResourceFile_IO.BuildInstrumentResourceFile(FullFileName: string;
  aInstrument: TInstrument);
begin

  lvStringList := TStringList.Create;

  with aInstrument do
    begin
      lvStringList.Add(' Instrument follows');
      lvStringList.Add(
        InstrumentName + U +
        PrintTitle + U +
        Version + U +
        BoolToStr(RequiresSignature) + U +
        Reference + U +
        BoolToStr(RequiresLicense) + U +
        IsOperational + U +
        BoolToStr(LicenseIsCurrent) + U +
        BoolToStr(IsLegacy) + U +
        IntToStr(NumberOfQuestions) + U +
        IEN + U +
        Publisher + U +
        Author + U +
        Copyright + U +
        PublicationDate + U +
        NormSample + U +
        R_Privilege + U +
        Purpose + U +
        TargetPopulation + U +
        EntryDate + U +
        EnteredBy + U +
        A_Privilege + U +
        LastEditedBy + U +
        LastEditDate + U +
        BoolToStr(IsNationalTest) + U +
        LastEditFMDate + U +
        BoolToStr(SubmitToNationalDB) + U +
        BoolToStr(IsCopyrighted) + U +
        BoolToStr(WriteFullText) + U +
        IntToStr(DaysToRestart) + U +
        BoolToStr(GenerateProgressNote) + U +
        TIU_Title + U +
        Consult_Title + U +
        DLL_Version + U +
        DLL_Date);

      if DLL_Version <> '' then
        UpdateLatestVersion(DLL_Version);
    end;

  lvStringList.Add('End Instrument');
  lvStringList.Add('');

  lvStringList.Add('Questions follow');
  WriteQuestions(aInstrument);
  lvStringList.Add('End Questions');
  lvStringList.Add('');

  lvStringList.Add('Introductions follow');
  WriteIntroductions(aInstrument);
  lvStringList.Add('End Introductions');
  lvStringList.Add('');

  lvStringList.Add('Sections follow');
  WriteSections(aInstrument);
  lvStringList.Add('End Sections');
  lvStringList.Add('');

  lvStringList.Add('Rules follow');
  WriteRules(aInstrument);
  lvStringList.Add('End Rules');
  lvStringList.Add('');

  lvStringList.Add('End of Write file');
  lvStringList.SaveToFile(FullFileName);

  lvStringList.Free;
end;

procedure TResourceFile_IO.ReadInstrumentResourceFile(FullFileName: string;
  var aInstrument: TInstrument);
begin
  lvStringList := TStringList.Create;

  lvFont := TFont.Create;
  lvStringList.LoadFromFile(FullFileName);

  lvStartRule := 0;
  lvStartIntro := 0;
  lvStartSection := 0;

  ReadDescriptiveProperties(aInstrument);

  aInstrument.ListOfQuestions.Clear;
  aInstrument.ListOfIntroductions.Clear;
  aInstrument.ListOfSections.Clear;

  ReadQuestions(aInstrument);
  ReadIntroductions(aInstrument);

  ReadSections(aInstrument);
  ReadRules(aInstrument);
end;

function TResourceFile_IO.ReadInstrumentDetailsFromFile(FullFileName: string): string;
// File has already been checked.
// Yields string with DaysToRestart: integer (Range is -1 to 99
//  where -1 is no limit, 0 is not allowed) ^
// GenerateProgNote: Boolean ^
// ProgNoteTitle: string ^
// ConsultTitle: string
begin
  lvStringList := TStringList.Create;
  lvStringList.LoadFromFile(FullFileName);

//  Result := StrToIntDef(Piece(lvStringList[1], U, 30), 2);
  Result := Piece(lvStringList[1], U, 30) + U +
    Piece(lvStringList[1], U, 31) + U +
    Piece(lvStringList[1], U, 32) + U +
    Piece(lvStringList[1], U, 33);

  lvStringList.Free;
end;

function ConvertChoiceIdentifierToString
  (ChoiceIdent: TChoiceIdentifier): string;
begin
  Result := 'ONE';
  if ChoiceIdent = ciZeroBased then
    Result := 'ZERO';
  if ChoiceIdent = ciNone then
    Result := 'NONE';
end;

function ConvertResponseTypeToString(AResponseType: TResponseType): string;
begin
  Result := String_RT; //make this the default
  if (AResponseType = rtMultiChoice) then
    Result := MCHOICE_RT;
  if (AResponseType = rtInteger) then
    Result := INTEGER_RT;
  if (AResponseType = rtDate) then
    Result := DATE_RT;
  if (AResponseType = rtMemo) then
    Result := MEMO_RT;
  if (AResponseType = rtCurrency) then
    Result := CURRENCY_RT;
  if (AResponseType = rtVista) then
    Result := VISTA_RT;
  if (AResponseType = rtMaskEdit) then
    Result := MASKEDIT_RT;
  if (AResponseType = rtTrackBar) then
    Result := TRACKBAR_RT;
  if (AResponseType = rtCheckList) then
    Result := CHECKLIST_RT;
end; //ConvertToResponseType

function StyleToString(StyleIn: TFontStyles): string;
begin
  Result := '';
  if fsBold in StyleIn then
    Result := 'Y' + U
  else
    Result := 'N' + U;
  if fsItalic in StyleIn then
    Result := Result + 'Y' + U
  else
    Result := Result + 'N' + U;
  if fsUnderline in StyleIn then
    Result := Result + 'Y' + U
  else
    Result := Result + 'N' + U;
end;

function AlignmentToStr(AAlignment: TAlignment): string;
begin
  case AAlignment of
    taLeftJustify: Result := LEFT_JUSTIFY;
    taCenter: Result := CENTER_JUSTIFY;
    taRightJustify: Result := RIGHT_JUSTIFY;
  end;
end;

function AnswerComponentToStr(AAnswerComponent: TAnswerComponent): string;
begin
  Result := 'NA';

  case AAnswerComponent of

    //DATA TYPE OF COMPONENT: SET OF CODES//
    //INTERNALLY-STORED CODE: RA//   WILL STAND FOR: RadioGroup//
    //INTERNALLY-STORED CODE: CK//   WILL STAND FOR: CheckList//
    //INTERNALLY-STORED CODE: CO//   WILL STAND FOR: ComboBox//
    //INTERNALLY-STORED CODE: LI//   WILL STAND FOR: List//
    //INTERNALLY-STORED CODE: ED//   WILL STAND FOR: Edit//
    //INTERNALLY-STORED CODE: MA//   WILL STAND FOR: Mask Edit//
    //INTERNALLY-STORED CODE: SP//   WILL STAND FOR: Spin Edit//
    //INTERNALLY-STORED CODE: ME//   WILL STAND FOR: Memo//
    //INTERNALLY-STORED CODE: DA//   WILL STAND FOR: Date Picker//
    //INTERNALLY-STORED CODE: TR//   WILL STAND FOR: Track Bar//
    //INTERNALLY-STORED CODE: OR//   WILL STAND FOR: ORcombo Box//
    //INTERNALLY-STORED CODE: OS//   WILL STAND FOR: Staff Combo Box//
    //INTERNALLY-STORED CODE: OC//   WILL STAND FOR: Consultation Combo Box//
    //INTERNALLY-STORED CODE: NA//   WILL STAND FOR: Not Applicable//

    acComboBox: Result := COMBOBOX_AC;
    acCheckListBox: Result := CHECKLIST_AC;
    acListBox: Result := LISTBOX_AC;
    acRadioGroup: Result := RADIOGROUP_AC;
    acEdit: Result := EDIT_AC;
    acMaskEdit: Result := MASKEDIT_AC;
    acSpinEdit: Result := SPINEDIT_AC;
    acMemo: Result := MEMO_AC;
    acDatePicker: Result := DATEPICKER_AC;
    acTrackBar: Result := TRACKBAR_AC;
    //acORCombo: Result := ORCOMBOBOX_AC;
    acORCombo: Result := ORSTAFFCOMBOBOX_AC;
    acStaff: Result := ORSTAFFCOMBOBOX_AC;
    acConsult: Result := ORCONSULTCOMBOBOX_AC;
  end;
end;


procedure TResourceFile_IO.WriteQuestions(aInstrument: TInstrument);
var
  i                           : Integer;

  function StringOfChoices(aChoices: TStringList): string;
  // Each line has: Sequence^ChoiceID^Text^Legacy Value
  var
    ii                        : Integer;
  begin
    Result := 'NONE';

    if aChoices.Count = 0 then
      Exit;

    Result := RECORD_END;

    for ii := 0 to aChoices.Count - 1 do
      Result := Result + aChoices[ii] + RECORD_END;
  end;

  function MultipleChoiceItems(DisplayComp: TAnswerComponent): string;
  begin
    Result := '';
    if (DisplayComp = acComboBox) or
      (DisplayComp = acListBox) or
      (DisplayComp = acRadioGroup) or
      (DisplayComp = acTrackBar) or
      (DisplayComp = acCheckListBox) then
      begin
        with TQuestion(aInstrument.ListOfQuestions[i]) do
          begin
            Result := 'Choices Follow ' + U +
              ConvertChoiceIdentifierToString(ChoiceIdentifier) + U +
              IntToStr(MultiChoice.NumberOfPicks) + U +
              IntToStr(MultiChoice.NumberOfChoices) + U +
              StringOfChoices(MultiChoice.Choices) + U +
              'End Choices';
          end;
      end;
  end;

begin
  if aInstrument.ListOfQuestions.Count = 0 then
    Exit;

  for i := 0 to aInstrument.ListOfQuestions.Count - 1 do
    begin
      with TQuestion(aInstrument.ListOfQuestions[i]) do
        begin
          lvStringList.Add(' ');
          lvStringList.Add('Question_' + IntToStr(i) + U +
            // Question properties
            qText + U +
            QuestionID + U +
            IntToStr(qMax) + U +
            BoolToStr(IsRequired) + U +
            IntToStr(qMin) + U +
            Designator + U +
            qHint + U +
            qCaption + U +
            //BoolToStr(IsLastQuestion) + U +
            //BoolToStr(IsFirstQuestion) + U +
            ConvertResponseTypeToString(ResponseType) + U +

            // Question Display
            Display.DisplayID + U +
            Display.DisplayFont.Name + U +
            StyleToString(Display.DisplayFont.Style) + // No U
            IntToStr(Display.DisplayFont.Size) + U +
            ColorToString(Display.DisplayFont.Color) + U +
            IntToStr(Display.DisplayLeft) + U +
            AlignmentToStr(Display.DisplayAlignment) + U +

            // Answer Display
            AnswerDisplay.DisplayID + U +
            AnswerDisplay.DisplayFont.Name + U +
            StyleToString(AnswerDisplay.DisplayFont.Style) + // No U
            IntToStr(AnswerDisplay.DisplayFont.Size) + U +
            ColorToString(AnswerDisplay.DisplayFont.Color) + U +
            IntToStr(AnswerDisplay.DisplayLeft) + U +
            IntToStr(AnswerDisplay.DisplayWidth) + U +
            AlignmentToStr(AnswerDisplay.DisplayAlignment) + U +
            AnswerDisplay.DisplayMask + U +
            IntToStr(AnswerDisplay.DisplayColumns) + U +
            AnswerComponentToStr(AnswerDisplay.DisplayComponent) + U +
            BoolToStr(AnswerDisplay.DisplaySameLine) + U +

            // Choices
            MultipleChoiceItems(AnswerDisplay.DisplayComponent));

          lvStringList.Add('End Question_' + IntToStr(i));
        end;
    end;
end;


procedure TResourceFile_IO.WriteIntroductions(aInstrument: TInstrument);
var
  i                           : Integer;
begin

  if aInstrument.ListOfIntroductions.Count = 0 then
    Exit;

  for i := 0 to aInstrument.ListOfIntroductions.Count - 1 do
    begin
      with TIntroduction(aInstrument.ListOfIntroductions[i]) do
        begin
          lvStringList.Add(' ');
          lvStringList.Add('Introduction_' + IntToStr(i) + U +
            ID + U +
            iText + U +
            BoolToStr(IsNew) + U +
            Display.DisplayID + U +
            Display.DisplayFont.Name + U +
            StyleToString(Display.DisplayFont.Style) + // No U
            IntToStr(Display.DisplayFont.Size) + U +
            ColorToString(Display.DisplayFont.Color) + U +
            IntToStr(Display.DisplayLeft) + U +
            AlignmentToStr(Display.DisplayAlignment) + U);
          lvStringList.Add('End Introduction_' + IntToStr(i));
        end; // with
    end;
end;


procedure TResourceFile_IO.WriteSections(aInstrument: TInstrument);
var
  i                           : Integer;
begin
  for i := 0 to aInstrument.ListOfSections.Count - 1 do
    begin
      with TSectionLabel(aInstrument.ListOfSections[i]) do
        begin
          lvStringList.Add(' ');
          lvStringList.Add('Section_' + IntToStr(i) + U +
            QuestionID + U +
            TabCaption + U +
            SectionCaption + U +
            Display.DisplayID + U +
            Display.DisplayFont.Name + U +
            StyleToString(Display.DisplayFont.Style) + // No U
            IntToStr(Display.DisplayFont.Size) + U +
            ColorToString(Display.DisplayFont.Color) + U +
            IntToStr(Display.DisplayLeft) + U +
            AlignmentToStr(Display.DisplayAlignment) + U);
          lvStringList.Add('End Section_' + IntToStr(i));
        end; // with
    end; // for i
end;

function DataTypeToString(ADataValueType: TValueDataType): string;
begin
  //  TValueDataType = (dtString, dtDate, dtNumber, dtTimeInterval);
  case ADataValueType of
    dtDate: Result := DATE_STR;
    dtNumber: Result := NUMBER_STR;
    dtTimeInterval: Result := INTERVAL_STR;
  else
    Result := STRING_STR;
  end; //case
end;

function QuestionOperatorToStr(ThisRule: TRule; IsFirstQuestion: BOOLEAN): string;
var
  tmpOperator                 : TOperatorType;
begin
  with ThisRule do
    begin
      if IsFirstQuestion then
        tmpOperator := FirstOperator
      else
        tmpOperator := SecondOperator;

      Result := '';

  //OPERATOR_STRINGS = 'Equals^Does not equal^Is less than^Is greater than^'
  //+ 'Equals or is less than^Equals or is greater than';

      case tmpOperator of
        toEQ: Result := Piece(OPERATOR_STRINGS, U, 1);
        toNE: Result := Piece(OPERATOR_STRINGS, U, 2);
        toLT: Result := Piece(OPERATOR_STRINGS, U, 3);
        toGT: Result := Piece(OPERATOR_STRINGS, U, 4);
        toLE: Result := Piece(OPERATOR_STRINGS, U, 5);
        toGE: Result := Piece(OPERATOR_STRINGS, U, 6);
      end;
    end;
end;


function BoolOperatorToString(ThisRule: TRule): string;
begin
  Result := 'AND';

  if (ThisRule.BoolOperator = boOR) then
    Result := 'OR';

end;


procedure TResourceFile_IO.WriteRules(aInstrument: TInstrument);
var
  i                           : Integer;
begin
  if aInstrument.ListOfRules.Count = 0 then
    Exit;

  for i := 0 to aInstrument.ListOfRules.Count - 1 do
    begin
      with TRule(aInstrument.ListOfRules[i]) do
        begin
          lvStringList.Add(' ');
          lvStringList.Add('Rule_' + IntToStr(i) + U +
            DataTypeToString(FirstValueDataType) + U +
            QuestionOperatorToStr(TRule(aInstrument.ListOfRules[i]), True) + U +
            DataTypeToString(SecondValueDataType) + U +
            QuestionOperatorToStr(TRule(aInstrument.ListOfRules[i]), False) + U +
            BoolOperatorToString(TRule(aInstrument.ListOfRules[i])) + U +
            FirstQuestionID + U +
            FirstValue + U +
            SecondQuestionID + U +
            SecondValue + U +
            MessageText + U +
            'Skip*' + SkipQuestionIDs);
        end;
    end; // for i

end;

(*
procedure TResourceFile_IO.WriteScales(aInstrument: TInstrument);
var
  i, j, k                     : Integer;
begin

  if aInstrument.ListOfScaleGroups.count = 0 then
    exit;

  for i := 0 to aInstrument.ListOfScaleGroups.Count - 1 do
    begin
      with TScaleGroup(aInstrument.ListOfScaleGroups[i]) do
        begin
          lvStringList.Add('');

          lvStringList.Add('ScaleGroup_' + IntToStr(i) + U +
            GroupName + U +
            IEN + U +
            OrdinateTitle + U +
            IntToStr(OrdinateMax) + U +
            IntToStr(OrdinateIncrement) + U +
            IntToStr(OrdinateMin) + U +
            IntToStr(Grid1Ordinate) + U +
            IntToStr(Grid2Ordinate) + U +
            IntToStr(Grid3Ordinate) + U +
            IntToStr(NumberOfScales) + U +
            IntToStr(Sequence));
          lvStringList.Add('End ScaleGroup_' + IntToStr(i));
        end;
    end;

  for i := 0 to aInstrument.ListOfScales.Count - 1 do
    begin
      with TScale(aInstrument.ListOfScales[i]) do
        begin
          lvStringList.Add('');

          lvStringList.Add('   Scale_' + IntToStr(i) + U +
            ScaleName + U +
            IEN + U +
            GroupIEN + U +
            XLabel + U +
            IntToStr(Sequence));
          lvStringList.Add('   End Scale_' + IntToStr(i));
        end;
    end;

  for i := 0 to aInstrument.ListOfKeys.Count - 1 do
    begin
      with TKey(aInstrument.ListOfKeys[i]) do
        begin
          lvStringList.Add('');

          lvStringList.Add('      ScoringKey_' + IntToStr(i) + U +
            IEN + U +
            QuestionID + U +
            ScaleID + U +
            AnswerText + U +
            Value);
          lvStringList.Add('   ScoringKey_' + IntToStr(i))
        end;
    end; // with TKey(Instrument.ListOfKeys[k])

end;   *)


procedure TResourceFile_IO.ReadDescriptiveProperties(aInstrument: TInstrument);
var
  i                           : Integer;
  TempStr                     : string;
begin
  with aInstrument do
    begin
      //Instrument_OPS.Clear;
      i := 1;
      InstrumentName := Piece(lvStringList[1], U, i);
      Inc(i); // 2
      PrintTitle := Piece(lvStringList[1], U, i);
      Inc(i);
      Version := Piece(lvStringList[1], U, i);
      Inc(i);
      RequiresSignature := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i); // 5
      Reference := Piece(lvStringList[1], U, i);
      Inc(i);
      RequiresLicense := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i);
      TempStr := Piece(lvStringList[1], U, i);
      IsOperational := TempStr[1];
      Inc(i); // 8
      LicenseIsCurrent := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i);
      IsLegacy := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i); // 10
      NumberOfQuestions := StrToIntDef(Piece(lvStringList[1], U, i), 0);
      Inc(i);
      IEN := Piece(lvStringList[1], U, i);
      Inc(i);
      Publisher := Piece(lvStringList[1], U, i);
      Inc(i);
      Author := Piece(lvStringList[1], U, i);
      Inc(i);
      Copyright := Piece(lvStringList[1], U, i);
      Inc(i); // 15
      PublicationDate := Piece(lvStringList[1], U, i);
      Inc(i);
      NormSample := Piece(lvStringList[1], U, i);
      Inc(i);
      R_Privilege := Piece(lvStringList[1], U, i);
      Inc(i);
      Purpose := Piece(lvStringList[1], U, i);
      Inc(i);
      TargetPopulation := Piece(lvStringList[1], U, i);
      Inc(i); // 20
      EntryDate := Piece(lvStringList[1], U, i);
      Inc(i);
      EnteredBy := Piece(lvStringList[1], U, i);
      Inc(i);
      A_Privilege := Piece(lvStringList[1], U, i);
      Inc(i);
      LastEditedBy := Piece(lvStringList[1], U, i);
      Inc(i);
      LastEditDate := Piece(lvStringList[1], U, i);
      Inc(i); // 25
      IsNationalTest := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i);
      LastEditFMDate := Piece(lvStringList[1], U, i);
      Inc(i);
      SubmitToNationalDB := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i);
      IsCopyrighted := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i);
      WriteFullText := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i); // 30
      DaysToRestart := StrToIntDef(Piece(lvStringList[1], U, i), 2);
      Inc(i); // 31
      GenerateProgressNote := StrToBool(Piece(lvStringList[1], U, i));
      Inc(i); // 32
      TIU_Title := Piece(lvStringList[1], U, i);
      Inc(i); // 33
      Consult_Title := Piece(lvStringList[1], U, i);

      Inc(i); // 34
      DLL_Version := Piece(lvStringList[1], U, i);
      Inc(i); // 35
      DLL_Date := Piece(lvStringList[1], U, i);

    end;
end;


procedure TResourceFile_IO.ReadQuestions(aInstrument: TInstrument);
var
  i, j, k                     : Integer;
  AQuestion                   : TQuestion;

begin
  with Display_OPS, Question_OPS do
    begin
  // Start a 3; 0,1,2 for instrument object
      for i := 2 to lvStringList.Count - 1 do
        begin

      // String with Question data
          if Pos('Question_', lvStringList[i]) = 1 then
            begin
              lvStartIntro := i;
              j := 2;
              AQuestion := TQuestion.Create;

              AQuestion.qText := Piece(lvStringList[i], U, j);
              Inc(j);
              AQuestion.QuestionID := Piece(lvStringList[i], U, j);
              AQuestion.qTag := StrToIntDef(AQuestion.QuestionID, -1);
              Inc(j);
              AQuestion.qMax := StrToIntDef(Piece(lvStringList[i], U, j), 0);
              Inc(j);
              AQuestion.IsRequired := StrToBool(Piece(lvStringList[i], U, j));
              Inc(j);
              AQuestion.qMin := StrToIntDef(Piece(lvStringList[i], U, j), 0);
              Inc(j);
              AQuestion.Designator := Piece(lvStringList[i], U, j);
              Inc(j);
              AQuestion.qHint := Piece(lvStringList[i], U, j);
              Inc(j);
              AQuestion.qCaption := Piece(lvStringList[i], U, j);
              Inc(j);
          //AQuestion.IsLastQuestion := StrToBool(Piece(lvStringList[i], U, j));
          //Inc(j);
          //AQuestion.IsFirstQuestion := StrToBool(Piece(lvStringList[i], U, j));
          //Inc(j);
              AQuestion.ResponseType := ConvertStringToResponseType
                (Piece(lvStringList[i], U, j));

              Inc(j);

          // Build Question Display
              AQuestion.Display.DisplayID := Piece(lvStringList[i], U, j);
              Inc(j);
              AQuestion.Display.DisplayFont.Name := Piece(lvStringList[i], U, j); // Font.name
              Inc(j);
              AQuestion.Display.DisplayFont.Style := [];
              if Piece(lvStringList[i], U, j) = 'Y' then // Bold: boolean
                AQuestion.Display.DisplayFont.Style := [fsBold];
              Inc(j);
              if Piece(lvStringList[i], U, j) = 'Y' then // Italic: boolean
                AQuestion.Display.DisplayFont.Style :=
                  AQuestion.Display.DisplayFont.Style + [fsItalic];
              Inc(j);
              if Piece(lvStringList[i], U, j) = 'Y' then // Underline: boolean
                AQuestion.Display.DisplayFont.Style :=
                  AQuestion.Display.DisplayFont.Style + [fsUnderline];
              Inc(j);
          //Size: integer
              AQuestion.Display.DisplayFont.Size := StrToIntDef(
                Piece(lvStringList[i], U, j), 8);
              Inc(j);
          //Color: string
              AQuestion.Display.DisplayFont.Color := StringToColor(
                Piece(lvStringList[i], U, j));
              Inc(j);
          //Left: integer
              AQuestion.Display.DisplayLeft :=
                StrToIntDef(Piece(lvStringList[i], U, j), 1);
              Inc(j);
          //Align: TAlignment
              AQuestion.Display.DisplayAlignment :=
                StrToAlignment(Piece(lvStringList[i], U, j));
              Inc(j);

          // Build AQuestion.AnswerDisplay
              AQuestion.AnswerDisplay.DisplayID := Piece(lvStringList[i], U, j); // ID
              Inc(j);
              AQuestion.AnswerDisplay.DisplayFont.Name := Piece(lvStringList[i], U, j); // Font.name
              Inc(j);
              AQuestion.AnswerDisplay.DisplayFont.Style := [];
              if Piece(lvStringList[i], U, j) = 'Y' then // Bold: boolean
                AQuestion.AnswerDisplay.DisplayFont.Style := [fsBold];
              Inc(j);
              if Piece(lvStringList[i], U, j) = 'Y' then // Italic: boolean
                AQuestion.AnswerDisplay.DisplayFont.Style :=
                  AQuestion.AnswerDisplay.DisplayFont.Style + [fsItalic];
              Inc(j);
              if Piece(lvStringList[i], U, j) = 'Y' then // Underline: boolean
                AQuestion.AnswerDisplay.DisplayFont.Style :=
                  AQuestion.AnswerDisplay.DisplayFont.Style + [fsUnderline];
              Inc(j);
          //Size: integer
              AQuestion.AnswerDisplay.DisplayFont.Size := StrToIntDef(
                Piece(lvStringList[i], U, j), 8);
              Inc(j);
          //Color: string
              AQuestion.AnswerDisplay.DisplayFont.Color := StringToColor(
                Piece(lvStringList[i], U, j));
              Inc(j);
          //Left: integer
              AQuestion.AnswerDisplay.DisplayLeft := StrToIntDef(
                Piece(lvStringList[i], U, j), 1);
              Inc(j);
          // Width : integer
              AQuestion.AnswerDisplay.DisplayWidth := StrToIntDef(
                Piece(lvStringList[i], U, j), 100);
              Inc(j);
          //Align: TAlignment
              AQuestion.AnswerDisplay.DisplayAlignment :=
                StrToAlignment(
                Piece(lvStringList[i], U, j));
              Inc(j);
              AQuestion.AnswerDisplay.DisplayMask := Piece(lvStringList[i], U, j);
              Inc(j);
              AQuestion.AnswerDisplay.DisplayColumns := StrToIntDef(
                Piece(lvStringList[i], U, j), 1);
              Inc(j);
              AQuestion.AnswerDisplay.DisplayComponent :=
                StrToAnswerComponent(
                Piece(lvStringList[i], U, j));
              Inc(j);
              AQuestion.AnswerDisplay.DisplaySameLine :=
                StrToBool(Piece(lvStringList[i], U, j));
              Inc(j);

          // Choices
              if Pos('Choices Follow', lvStringList[i]) > 0 then
                begin
                  Inc(j);
                  AQuestion.ChoiceIdentifier :=
                    ConvertStringToChoiceIdentifier(Piece(lvStringList[i], U, j));
                  Inc(j);
                  AQuestion.MultiChoice.NumberOfPicks := StrToInt(Piece(lvStringList[i], U, j));
                  Inc(j);
                  AQuestion.MultiChoice.NumberOfChoices := StrToInt(Piece(lvStringList[i], U, j));

                  if AQuestion.MultiChoice.NumberOfChoices > 0 then
                    for k := 2 to AQuestion.MultiChoice.NumberOfChoices + 1 do
                      AQuestion.MultiChoice.Choices.Add(Piece(lvStringList[i], RECORD_END, k));
                end;

// ShowMessage('AQuestion.MultiChoice.NumberOfPicks '+ IntToStr(AQuestion.MultiChoice.NumberOfPicks) +#13+
// 'AQuestion.MultiChoice.NumberOfChoices ' +IntToStr(AQuestion.MultiChoice.NumberOfChoices));
// if AQuestion.MultiChoice.Choices.Count > 1 then
// ShowMessage(
// 'AQuestion.MultiChoice.Choices[0] ' +   AQuestion.MultiChoice.Choices[0] +#13+
// 'AQuestion.MultiChoice.Choices[1] ' +  AQuestion.MultiChoice.Choices[1]);

              aInstrument.ListOfQuestions.Add(AQuestion);
            end; // String with question data

        end; // for i; each string in import file
    end; // with Display_OPS
end;


procedure TResourceFile_IO.ReadIntroductions(aInstrument: TInstrument);
var
  i, j                        : Integer;
  AIntroduction               : TIntroduction;
begin
  with Display_OPS do
    begin
      for i := lvStartIntro to lvStringList.Count - 1 do
        begin

          if Pos('Introduction_', lvStringList[i]) = 1 then
            begin
              with AIntroduction do
                begin
                  AIntroduction := TIntroduction.Create;
                  lvStartSection := i;
                  j := 2;

                  ID := Piece(lvStringList[i], U, j);
                  Inc(j);
                  iText := Piece(lvStringList[i], U, j);
                  Inc(j);
                  IsNew := StrToBool(Piece(lvStringList[i], U, j));
                  Inc(j);

                  Display.DisplayID := Piece(lvStringList[i], U, j); // ID
                  Inc(j);
                  Display.DisplayFont.Name := Piece(lvStringList[i], U, j); // Font.name
                  Inc(j);
                  Display.DisplayFont.Style := [];
                  if Piece(lvStringList[i], U, j) = 'Y' then // Bold: boolean
                    Display.DisplayFont.Style := [fsBold];
                  Inc(j);
                  if Piece(lvStringList[i], U, j) = 'Y' then // Italic: boolean
                    Display.DisplayFont.Style :=
                      Display.DisplayFont.Style + [fsItalic];
                  Inc(j);
                  if Piece(lvStringList[i], U, j) = 'Y' then // Underline: boolean
                    Display.DisplayFont.Style :=
                      Display.DisplayFont.Style + [fsUnderline];
                  Inc(j);
              //Size: integer
                  Display.DisplayFont.Size := StrToIntDef(
                    Piece(lvStringList[i], U, j), 8);
                  Inc(j);
              //Color: string
                  Display.DisplayFont.Color := StringToColor(
                    Piece(lvStringList[i], U, j));
                  Inc(j);
               //Left: integer
                  Display.DisplayLeft := StrToIntDef(
                    Piece(lvStringList[i], U, j), 1);
                  Inc(j);
              //Align: TAlignment
                  Display.DisplayAlignment :=
                    StrToAlignment(Piece(lvStringList[i], U, j));

                  aInstrument.ListOfIntroductions.Add(AIntroduction);

                end;
            end;
        end;
    end; // with Display_OPS
end;


procedure TResourceFile_IO.ReadSections(aInstrument: TInstrument);
var
  i, j                        : Integer;
  ASection                    : TSectionLabel;

begin

  // Just in case there are no introductions
  if lvStartSection = 0 then
    lvStartSection := lvStartIntro;
  with Display_OPS do
    begin
      for i := lvStartSection to lvStringList.Count - 1 do
        begin

          if Pos('Section_', lvStringList[i]) = 1 then
            begin
              ASection := TSectionLabel.Create;
              with ASection do
                begin
                  lvStartRule := i;
                  j := 2;

                  QuestionID := Piece(lvStringList[i], U, j);
                  Inc(j);
                  TabCaption := Piece(lvStringList[i], U, j);
                  Inc(j);
                  SectionCaption := Piece(lvStringList[i], U, j);
                  Inc(j);
                  Display.DisplayID := Piece(lvStringList[i], U, j); // ID
                  Inc(j);
                  Display.DisplayFont.Name := Piece(lvStringList[i], U, j); // Font.name
                  Inc(j);
                  Display.DisplayFont.Style := [];
                  if Piece(lvStringList[i], U, j) = 'Y' then // Bold: boolean
                    Display.DisplayFont.Style := [fsBold];
                  Inc(j);
                  if Piece(lvStringList[i], U, j) = 'Y' then // Italic: boolean
                    Display.DisplayFont.Style :=
                      Display.DisplayFont.Style + [fsItalic];
                  Inc(j);
                  if Piece(lvStringList[i], U, j) = 'Y' then // Underline: boolean
                    Display.DisplayFont.Style :=
                      Display.DisplayFont.Style + [fsUnderline];
                  Inc(j);
              //Size: integer
                  Display.DisplayFont.Size := StrToIntDef(
                    Piece(lvStringList[i], U, j), 8);
                  Inc(j);
              //Color: string
                  Display.DisplayFont.Color := StringToColor(
                    Piece(lvStringList[i], U, j));
                  Inc(j);
               //Left: integer
                  Display.DisplayLeft := StrToIntDef(
                    Piece(lvStringList[i], U, j), 1);
                  Inc(j);
              //Align: TAlignment
                  Display.DisplayAlignment :=
                    StrToAlignment(Piece(lvStringList[i], U, j));

                  aInstrument.ListOfSections.Add(ASection);

                end; // POS
            end; // with
        end; // for i
    end; // with Display_OPS
end;


procedure TResourceFile_IO.ReadRules(aInstrument: TInstrument);
var
  x, i, j                     : Integer;
  ARule                       : TRule;

  function MyStrToBoolOp(ABoolOperatorString: string): TBoolOperatorType;
  begin
    Result := boAND; //the default value
    if (ABoolOperatorString = 'OR') then
      Result := boOR;
  end; //MyStrToBoolOp

  function MyStrToDataType(ADataTypeString: string): TValueDataType;
  begin
    //dtString, dtDate, dtNumber
    Result := dtString;
    if (ADataTypeString = DATE_STR) then
      Result := dtDate;
    if (ADataTypeString = NUMBER_STR) then
      Result := dtNumber;
    if (ADataTypeString = INTERVAL_STR) then
      Result := dtTimeInterval;
  end; //MyStrToDataType

  function MyStrToOperator(AOperatorString: string): TOperatorType;
  var
    tmpIndex                  : integer;
    tmpString                 : string;
  begin
    //toEQ, toLT, toLE, toGT, toGE, toNE
   //  OPERATOR_STRINGS = 'Equals^Does not equal^Is less than^Is greater than^'
   //    + 'Equals or is less than^Equals or is greater than';

    tmpString := AOperatorString;
    tmpIndex := PieceIndex(OPERATOR_STRINGS, tmpString, U);

    case tmpIndex of
      2: Result := toNE;
      3: Result := toLT;
      4: Result := toGT;
      5: Result := toLE;
      6: Result := toGE;
    else
      Result := toEQ;
    end; //case

  end; //MyStrToSecondOp

begin
  // Just in case there are no Intro's or Sections
  if lvStartRule = 0 then
    lvStartRule := lvStartSection;
  if lvStartRule = 0 then
    lvStartRule := lvStartIntro;

  for i := lvStartRule to lvStringList.Count - 1 do
    begin

      // String with Rule data
      if Pos('Rule_', lvStringList[i]) = 1 then
        begin
          j := 2;
          ARule := TRule.Create;

          ARule.FirstValueDataType := MyStrToDataType(Piece(lvStringList[i], U, j));
          Inc(j);
          ARule.FirstOperator := MyStrToOperator(Piece(lvStringList[i], U, j));
          Inc(j);
          ARule.SecondValueDataType := MyStrToDataType(Piece(lvStringList[i], U, j));
          Inc(j);
          ARule.SecondOperator := MyStrToOperator(Piece(lvStringList[i], U, j));
          Inc(j);
          ARule.BoolOperator := MyStrToBoolOp(Piece(lvStringList[i], U, j));
          Inc(j);
          ARule.FirstQuestionID := Piece(lvStringList[i], U, j);
          Inc(j);
          ARule.FirstValue := Piece(lvStringList[i], U, j);
          Inc(j);
          ARule.SecondQuestionID := Piece(lvStringList[i], U, j);
          Inc(j);
          ARule.SecondValue := Piece(lvStringList[i], U, j);
          Inc(j);
          ARule.MessageText := Piece(lvStringList[i], U, j);

          x := Pos('Skip*', lvStringList[i]);

          ARule.SkipQuestionIDs := U + Copy(lvStringList[i], x + 5, Length(lvStringList[i]));

          aInstrument.ListOfRules.Add(ARule);
        end;
    end;
end;

(*
procedure TResourceFile_IO.ReadScales(aInstrument: TInstrument);
var
  i, j                        : Integer;
  TempScaleGroupID,
    TempScaleID,
    Reply                     : string;
  AScaleGroup                 : TScaleGroup;
  AScale                      : TScale;
  AKey                        : TKey;

begin

  for i := 2 to lvStringList.Count - 1 do
    begin

      // String with Scale Group data
      if Pos('ScaleGroup_', lvStringList[i]) = 1 then
        begin
          AScaleGroup := TScaleGroup.Create;
          j := 2;

          AScaleGroup.GroupName := Piece(lvStringList[i], U, j);
          Inc(j);
          AScaleGroup.IEN := Piece(lvStringList[i], U, j);
          Inc(j);
          AScaleGroup.OrdinateTitle := Piece(lvStringList[i], U, j);
          Inc(j);
          AScaleGroup.OrdinateMax := StrToIntDef(Piece(lvStringList[i], U, j), 10);
          Inc(j);
          AScaleGroup.OrdinateIncrement := StrToIntDef(Piece(lvStringList[i], U, j), 1);
          Inc(j);
          AScaleGroup.OrdinateMin := StrToIntDef(Piece(lvStringList[i], U, j), 0);
          Inc(j);
          AScaleGroup.Grid1Ordinate := StrToIntDef(Piece(lvStringList[i], U, j), 0);
          Inc(j);
          AScaleGroup.Grid2Ordinate := StrToIntDef(Piece(lvStringList[i], U, j), 0);
          Inc(j);
          AScaleGroup.Grid3Ordinate := StrToIntDef(Piece(lvStringList[i], U, j), 0);
          Inc(j);
          AScaleGroup.NumberOfScales := StrToIntDef(Piece(lvStringList[i], U, j), 1);
          Inc(j);
          AScaleGroup.Sequence := StrToIntDef(Piece(lvStringList[i], U, j), 1);

          aInstrument.ListOfScaleGroups.Add(AScaleGroup);
        end; // POS ScaleGroup

      // String with Scale data
      if Pos('Scale_', lvStringList[i]) > 0 then
        begin
          AScale := TScale.Create;
          j := 2;

          AScale.ScaleName := Piece(lvStringList[i], U, j);
          Inc(j);
          AScale.IEN := Piece(lvStringList[i], U, j);
          Inc(j);
          AScale.GroupIEN := Piece(lvStringList[i], U, j);
          Inc(j);
          AScale.XLabel := Piece(lvStringList[i], U, j);
          Inc(j);
          AScale.Sequence := StrToIntDef(Piece(lvStringList[i], U, j), 0);

          aInstrument.ListOfScales.Add(AScale);
        end;

      // String with Scoring Key data
      if Pos('ScoringKey_', lvStringList[i]) > 0 then
        begin
          AKey := TKey.Create;
          j := 2;

          AKey.IEN := Piece(lvStringList[i], U, j);
          Inc(j);
          AKey.QuestionID := Piece(lvStringList[i], U, j);
          Inc(j);
          AKey.ScaleID := Piece(lvStringList[i], U, j);
          Inc(j);
          AKey.AnswerText := Piece(lvStringList[i], U, j);
          Inc(j);
          AKey.Value := Piece(lvStringList[i], U, j);
          aInstrument.ListOfKeys.Add(AKey);

        end;
    end; // for i ...

end;  *)

end.

